from threading import Thread, Condition
import queue
import time


class BackgroundSaver:

    __instance = None

    def __init__(self):
        self.q = queue.Queue()
        self.condition = Condition()
        self.t = Thread(target=self.worker)
        self.t.start()


    def __new__(cls):
        if cls.__instance is None:
            cls.__instance = super().__new__(cls)
        return cls.__instance


    def add(self, function):
        with self.condition:
            self.q.put(function)
            self.condition.notify_all()


    def worker(self):
        while True:
            with self.condition:
                if not self.q.empty():
                    function = self.q.get()
                else:
                    self.condition.wait()
                    continue

            function()  # run outside of condition to prevent blocking while saving
