import sys
import math
import numpy as np

# import spectrum driver functions
from .pyspcm import *


def szTypeToName(lCardType):
    sName = ''
    lVersion = (lCardType & TYP_VERSIONMASK)
    if (lCardType & TYP_SERIESMASK) == TYP_M2ISERIES:
        sName = 'M2i.%04x'%lVersion
    elif (lCardType & TYP_SERIESMASK) == TYP_M2IEXPSERIES:
        sName = 'M2i.%04x-Exp'%lVersion
    elif (lCardType & TYP_SERIESMASK) == TYP_M3ISERIES:
        sName = 'M3i.%04x'%lVersion
    elif (lCardType & TYP_SERIESMASK) == TYP_M3IEXPSERIES:
        sName = 'M3i.%04x-Exp'%lVersion
    elif (lCardType & TYP_SERIESMASK) == TYP_M4IEXPSERIES:
        sName = 'M4i.%04x-x8'%lVersion
    elif (lCardType & TYP_SERIESMASK) == TYP_M4XEXPSERIES:
        sName = 'M4x.%04x-x4'%lVersion
    else:
        sName = 'unknown type'
    return sName


def get_hex_channels(n_channels, channels):
    if n_channels == 8:
        return CHANNEL0 | CHANNEL1 | CHANNEL2 | CHANNEL3 | CHANNEL4 | CHANNEL5 | CHANNEL6 | CHANNEL7
    if n_channels in {5, 6, 7}:
        sys.stdout.write("8 channels will be set.\n")
        return CHANNEL0 | CHANNEL1 | CHANNEL2 | CHANNEL3 | CHANNEL4 | CHANNEL5 | CHANNEL6 | CHANNEL7
    if n_channels == 4:
        return CHANNEL0 | CHANNEL1 | CHANNEL2 | CHANNEL3
    elif n_channels == 3:
        sys.stdout.write("4 channels will be set.\n")
        return CHANNEL0 | CHANNEL1 | CHANNEL2 | CHANNEL3
    elif n_channels == 2:
        if channels == None:
            return CHANNEL0 | CHANNEL1
        elif len(channels) == 2:
            return _get_hex_channels(channels)
        else:
            sys.stdout.write("Specify 2 channels.\n")

    elif n_channels == 1:
        if channels == None:
            return CHANNEL0
        elif len(channels) == 1:
            return _get_hex_channels(channels)
        else:
            sys.stdout.write("Specify 1 channel.\n")
    else:
        sys.stdout.write("Specify a valid number of channels.\n")


def _get_hex_channels(channels):
    if channels in {(0, 1), (1, 0)}:
        return CHANNEL0 | CHANNEL1
    elif channels in {(0, 2), (2, 0)}:
        return CHANNEL0 | CHANNEL2
    elif channels in {(0, 3), (3, 0)}:
        return CHANNEL0 | CHANNEL3
    elif channels in {(1, 2), (2, 1)}:
        return CHANNEL1 | CHANNEL2
    elif channels in {(1, 3), (3, 1)}:
        return CHANNEL1 | CHANNEL3
    elif channels in {(2, 3), (3, 2)}:
        return CHANNEL2 | CHANNEL3
    elif channels in {0, (0,)}:
        return CHANNEL0
    elif channels in {1, (1,)}:
        return CHANNEL1
    elif channels in {2, (2,)}:
        return CHANNEL2
    elif channels in {3, (3,)}:
        return CHANNEL3
    else:
        sys.stdout.write("Specify valid channels.\n")


def get_bin_path(path):
    if path == 'buffered':
        return 0
    elif path in {'HF', 'high frequency'}:
        return 1
    else:
        sys.stdout.write("Specify valid path.\n")


def get_bin_coupling(coupling):
    if coupling == 'DC':
        return 0
    elif coupling == 'AC':
        return 1
    else:
        sys.stdout.write("Specify valid coupling.\n")


def get_bin_bandwidth(bandwidth):
    if bandwidth == 'full':
        return 0
    elif bandwidth in {'limit', 'limited'}:
        return 1
    else:
        sys.stdout.write("Specify valid bandwidth.\n")


def get_bin_termination(termination):
    if termination == '50':
        return 1
    elif termination == '1M':
        return 0
    else:
         sys.stdout.write("Specify valid termination.\n")


def get_hex_trig_channel(channel):
    if channel == 0:
        return SPC_TMASK0_CH0
    elif channel ==1:
        return SPC_TMASK0_CH1
    elif channel ==2:
        return SPC_TMASK0_CH2
    elif channel ==3:
        return SPC_TMASK0_CH3
    else:
        sys.stdout.write("Specify valid channel.\n")


def get_hex_trig_mode(mode):
    if mode in {'pos', 'positive'}:
        return SPC_TM_POS
    if mode in {'neg', 'negative'}:
        return SPC_TM_NEG
    if mode in {'both'}:
        return SPC_TM_BOTH
    if mode in {'leave', 'leaving'}:
        return SPC_TM_WINLEAVE
    if mode in {'enter', 'entering'}:
        return SPC_TM_WINENTER
    else:
        sys.stdout.write("Specify valid trigger mode.\n")


def mvolt_to_digitized_level(mvolt, mv_range, bits=16):
    max_value = 2 ** (bits - 1)
    level = round(mvolt / mv_range * max_value).astype(int)

    if level > max_value:
        level = max_value
    elif level < - max_value:
        level = - max_value

    return int32(level)
