import numpy as np
import matplotlib.pyplot as plt
from matplotlib.path import Path
from matplotlib import patches


class Gate:
    
    def __init__(self, patch):
        self.patch = patch
        self.closed = False


    def connect(self):
        fig = self.patch.figure
        self.cidclick = fig.canvas.mpl_connect('button_press_event', self.onclick)
        self.cidleave = fig.canvas.mpl_connect('axes_leave_event', self.onleave)


    def onclick(self, event):
        if (event.inaxes != self.patch.axes) or self.closed: return

        path = self.patch.get_path()
        verts = path.vertices
        if np.array_equal(verts, np.zeros((1, 2))):
            verts = [[event.xdata, event.ydata]]
        else:
            verts = np.append(verts, [[event.xdata, event.ydata]], axis=0)
        path.vertices = verts


    def onleave(self, event):
        path = self.patch.get_path()
        verts = path.vertices

        if np.array_equal(verts, np.zeros((1, 2))):
            return

        path.vertices = np.append(verts, [verts[0]], axis=0)
        self.closed = True


    def get_path(self):
        return self.patch.get_path()


    def get_gate(self, x, y, gate=None):
        points = np.stack([x, y], axis=-1)
        path = self.patch.get_path()
        if gate is not None:
            new_gate = path.contains_points(points) & gate
        else:
            new_gate = path.contains_points(points)
        return new_gate
