import numpy as np
import re
import os


def array_to_dict(keys, array, axis):
    data_dict = dict()
    channels = array.shape[axis]
    if not channels == len(keys):
        print('Number of channel names does not mach array size.')
        print('Data will be set as single array.')
        return ['data'], array
    for ch in range(channels):
        key = keys[ch]
        data_dict.update({key: np.take(array, ch, axis=axis)})
    return data_dict


def set_attr(obj, key, value):
    setattr(obj, valid_attr_key(key), value)


def get_attr(obj, key, default=None):
    return getattr(obj, valid_attr_key(key), default)


def valid_attr_keys(keys):
    if type(keys) is list:
        new_keys = []
        for k in keys:
            new_keys.append(valid_attr_key(k))
    elif type(keys) is set:
        new_keys = set()
        for k in keys:
            new_keys.add(valid_attr_key(k))

    return new_keys


def valid_attr_key(key):
    return re.sub(r'[-\s]', '_', key).lower()


def recur_all_files(directory):
    for root, dirs, files in os.walk(directory):
        yield root
        for file in files:
            yield os.path.join(root, file)


def data_select(path, re=None):
    '''
    data selecter by name.
    '''
    all_files = [f for f in recur_all_files(path)]
    print('# of all files: ', len(all_files))
    if re is None:
        print('use all files')
        return all_files
    else:
        selected_files = [f for f in all_files if (re in f) and ('npz' in f)]
        print('# of selected files: ', len(selected_files))
    return selected_files


def smooth_reduce(array, n):
    return np.mean(np.reshape(array, (array.shape[0], -1, n)), axis=2)


def get_gate(path, x, y):
    '''
    path: matplotlibのpatchが作ってくれるpath
    '''
    points = np.stack([x, y], axis=-1)
    return path.contains_points(points)

